from ftw.solr.connection import SolrResponse
from ftw.solr.tests.utils import get_data
import socket
import unittest


class TestResponse(unittest.TestCase):

    def test_response_from_exception(self):
        exception = socket.error(61, 'Connection refused')
        resp = SolrResponse(exception=exception)
        self.assertFalse(resp.is_ok())
        self.assertEqual(resp.exception, exception)

    def test_response_from_bad_body(self):
        resp = SolrResponse(body='bad', status=200)
        self.assertFalse(resp.is_ok())
        self.assertEqual(resp.status, -1)

    def test_response_from_bad_request(self):
        body = get_data('bad_request.json')
        resp = SolrResponse(body=body, status=400)
        self.assertFalse(resp.is_ok())
        self.assertEqual(resp.http_status, 400)
        self.assertEqual(resp.status, 400)
        self.assertEqual(resp[u'error'][u'code'], 400)

    def test_response_from_search_request(self):
        body = get_data('search.json')
        resp = SolrResponse(body=body, status=200)
        self.assertTrue(resp.is_ok())
        self.assertEqual(len(resp.docs), 3)
        self.assertEqual(resp.num_found, 3)
        self.assertEqual(resp.start, 0)

    def test_response_from_search_with_highlighting(self):
        body = get_data('highlighting.json')
        resp = SolrResponse(body=body, status=200)
        self.assertTrue(resp.is_ok())
        self.assertIn('highlighting', resp)

    def test_response_from_schema_request(self):
        body = get_data('schema.json')
        resp = SolrResponse(body=body, status=200)
        self.assertTrue(resp.is_ok())
        self.assertIn('schema', resp)



# -*- coding: utf-8 -*-
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer

import ftw.solr


class FtwSolrLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        # import plone.app.dexterity
        # self.loadZCML(package=plone.app.dexterity)
        self.loadZCML(package=ftw.solr)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.solr:default')


FTW_SOLR_FIXTURE = FtwSolrLayer()


FTW_SOLR_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_SOLR_FIXTURE,),
    name='FtwSolrLayer:IntegrationTesting'
)


FTW_SOLR_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_SOLR_FIXTURE,),
    name='FtwSolrLayer:FunctionalTesting'
)



from ftw.solr.interfaces import ISolrConnectionManager
from ftw.solr.interfaces import ISolrIndexHandler
from ftw.solr.interfaces import ISolrIndexQueueProcessor
from logging import getLogger
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.interface import implementer


logger = getLogger('ftw.solr.indexer')


@implementer(ISolrIndexQueueProcessor)
class SolrIndexQueueProcessor(object):
    """A queue processor for solr """

    _manager = None

    def index(self, obj, attributes=None):
        """Index the given object."""
        handler = getMultiAdapter((obj, self.manager), ISolrIndexHandler)
        handler.add(attributes)

    def reindex(self, obj, attributes=None):
        """Reindex the given object."""
        self.index(obj, attributes)

    def unindex(self, obj):
        """Unindex the given object."""
        handler = getMultiAdapter((obj, self.manager), ISolrIndexHandler)
        handler.delete()

    def begin(self):
        """Called before processing of the queue is started."""
        pass

    def commit(self):
        """Called after processing of the queue has ended."""
        conn = self.manager.connection
        if conn is None:
            return
        conn.commit()

    def abort(self):
        """Called if processing of the queue needs to be aborted."""
        conn = self.manager.connection
        if conn is None:
            return
        conn.abort()

    @property
    def manager(self):
        if self._manager is None:
            self._manager = queryUtility(ISolrConnectionManager)
        return self._manager



# -*- coding: utf-8 -*-
from ftw.solr.indexer import SolrIndexQueueProcessor
from ftw.solr.interfaces import ISolrConnectionManager
from ftw.solr.interfaces import ISolrIndexHandler
from mock import MagicMock
from mock import PropertyMock
from plone.testing import zca
from zope.component import provideAdapter
from zope.component import provideUtility
from zope.interface import Interface
import unittest


class MockSolrIndexHandler(object):

    add = MagicMock(name='add')
    delete = MagicMock(name='delete')

    def __init__(self, context, manager):
        self.manger = manager


class TestSolrIndexQueueProcessor(unittest.TestCase):

    layer = zca.UNIT_TESTING

    def setUp(self):
        self.indexer = SolrIndexQueueProcessor()
        self.indexer._manager = MagicMock(name='SolrConnectionManager')
        provideAdapter(
            MockSolrIndexHandler,
            adapts=(Interface, MagicMock),
            provides=ISolrIndexHandler,
        )
        conn = MagicMock(name='SolrConnection')
        conn.commit = MagicMock(name='commit')
        type(self.indexer._manager).connection = PropertyMock(
            return_value=conn)

    def test_index_object_without_providing_attributes_calls_add(self):
        self.indexer.index(object())
        MockSolrIndexHandler.add.assert_called_with(None)

    def test_index_object_with_provided_attributes_calls_add(self):
        self.indexer.index(object(), attributes=['path'])
        MockSolrIndexHandler.add.assert_called_with(['path'])

    def test_reindex_object_without_providing_attributes_calls_add(self):
        self.indexer.reindex(object())
        MockSolrIndexHandler.add.assert_called_with(None)

    def test_reindex_object_with_provided_attributes_calls_add(self):
        self.indexer.reindex(object(), attributes=['path', 'Title'])
        MockSolrIndexHandler.add.assert_called_with(['path', 'Title'])

    def test_unindex_object_calls_delete(self):
        self.indexer.unindex(object())
        MockSolrIndexHandler.delete.assert_called_with()

    def test_begin_does_nothing(self):
        self.indexer.manager.connection.commit.assert_not_called()
        self.indexer.manager.connection.abort.assert_not_called()

    def test_commit_calls_connection_commit(self):
        self.indexer.commit()
        self.indexer.manager.connection.commit.assert_called_with()

    def test_abort_calls_connection_abort(self):
        self.indexer.abort()
        self.indexer.manager.connection.abort.assert_called_with()

    def test_manager_returns_registered_utility(self):
        self.indexer._manager = None
        util = object()
        provideUtility(util, ISolrConnectionManager)
        self.assertEqual(util, self.indexer.manager)



from ftw.solr.interfaces import ISolrConnectionManager
from ftw.solr.interfaces import ISolrIndexHandler
from logging import getLogger
from Products.CMFCore.interfaces import ICatalogAware
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import base_hasattr
from Products.Five.browser import BrowserView
from time import clock
from time import strftime
from time import time
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.component.hooks import getSite
import logging
import transaction


logger = getLogger('ftw.solr.maintenance')
logger.setLevel(logging.DEBUG)


def timer(func=time):
    """Set up a generator returning the elapsed time since the last call """
    def gen(last=func()):
        while True:
            elapsed = func() - last
            last = func()
            yield '%.3fs' % elapsed
    return gen()


def checkpoint_iterator(function, interval=100):
    """The iterator will call the given function for every nth invocation """
    counter = 0
    while True:
        counter += 1
        if counter % interval == 0:
            function()
        yield None


class SolrMaintenanceView(BrowserView):
    """Helper view for indexing content in Solr."""

    def __init__(self, context, request):
        super(SolrMaintenanceView, self).__init__(context, request)
        self.request.RESPONSE.setHeader('Content-Type', 'text/plain')

    def optimize(self):
        """Optimize the Solr index."""
        conn = self.manager.connection
        conn.optimize()
        return 'Solr index optimized.'

    def clear(self):
        """Clear all data from Solr index."""
        conn = self.manager.connection
        conn.delete_by_query('*:*')
        conn.commit()
        return 'Solr index cleared.'

    def reindex(self, commit_interval=100, idxs=None):
        """Reindex content in Solr."""

        processed = 0
        real = timer()
        lap = timer()
        cpu = timer(clock)

        transaction.doom()
        zodb_conn = self.context._p_jar

        def commit():
            conn = self.manager.connection
            conn.commit(extract_after_commit=False)
            zodb_conn.cacheGC()
            self.log(
                'Intermediate commit (%d items processed, last batch in %s)',
                processed, lap.next())

        cpi = checkpoint_iterator(commit, interval=commit_interval)
        self.log('Reindexing Solr...')
        for path, obj in find_objects(self.context):

            if not ICatalogAware.providedBy(obj):
                continue

            handler = getMultiAdapter((obj, self.manager), ISolrIndexHandler)
            handler.add(idxs)
            processed += 1
            cpi.next()

        commit()
        self.log('Solr index rebuilt.')
        self.log(
            'Processed %d items in %s (%s cpu time).',
            processed, real.next(), cpu.next())

    def reindex_cataloged(self, commit_interval=100, idxs=None, start=0,
                          end=-1):
        """Reindex all cataloged content in Solr."""
        site = getSite()
        catalog = getToolByName(self.context, 'portal_catalog')
        items = catalog.unrestrictedSearchResults(sort_on='path')

        try:
            start = int(start)
            end = int(end)
            commit_interval = int(commit_interval)
        except ValueError:
            start = 0
            end = -1
            commit_interval = 100

        processed = 0
        real = timer()
        lap = timer()
        cpu = timer(clock)

        transaction.doom()
        zodb_conn = self.context._p_jar

        def commit():
            conn = self.manager.connection
            conn.commit(extract_after_commit=False)
            zodb_conn.cacheGC()
            self.log(
                'Intermediate commit (%d items processed, last batch in %s)',
                processed, lap.next())

        cpi = checkpoint_iterator(commit, interval=commit_interval)
        self.log('Reindexing Solr...')
        for item in items[start:end]:
            path = item.getPath()
            obj = site.unrestrictedTraverse(path, None)
            if obj is None:
                logger.warning("Object at path %s doesn't exist", path)
                continue

            handler = getMultiAdapter((obj, self.manager), ISolrIndexHandler)
            handler.add(idxs)
            processed += 1
            cpi.next()

        commit()
        self.log('Solr index rebuilt.')
        self.log(
            'Processed %d items in %s (%s cpu time).',
            processed, real.next(), cpu.next())

    def log(self, msg, *args):
        logger.info(msg, *args)
        self.request.RESPONSE.write(
            strftime('%Y-%m-%d %H:%M:%S ') + msg % args + '\n')

    @property
    def manager(self):
        return queryUtility(ISolrConnectionManager)


def find_objects(context):
    """Generator to recursively find and yield all objects below the given
       context."""
    traverse = context.unrestrictedTraverse
    base = '/'.join(context.getPhysicalPath())
    cut = len(base) + 1
    paths = [base]
    for idx, path in enumerate(paths):
        obj = traverse(path)
        yield path[cut:], obj
        if base_hasattr(obj, 'objectIds'):
            for id_ in obj.objectIds():
                paths.insert(idx + 1, path + '/' + id_)



from ftw.solr.interfaces import ISolrSettings
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

import re


SPECIAL_CHARS = [
    '+', '-', '&&', '||', '!', '(', ')', '{', '}', '[', ']', '^', '"', '~',
    '*', '?', ':', '/']

OPERATORS = re.compile(r'(.*)\s+(AND|OR|NOT)\s+', re.UNICODE)


def escape(term):
    for char in SPECIAL_CHARS:
        term = term.replace(char, '\\' + char)
    return term


def is_simple_search(phrase):
    num_quotes = phrase.count('"')
    if num_quotes % 2 == 0:
        # Replace quoted parts with a marker
        # "foo bar" -> quoted
        parts = phrase.split('"')
        new_parts = []
        for i in range(0, len(parts)):
            if i % 2 == 0:
                new_parts.append(parts[i])
            else:
                new_parts.append('quoted')
        phrase = u''.join(new_parts)
    if bool(OPERATORS.match(phrase)):
        return False
    return True


def split_simple_search(phrase):
    parts = phrase.split('"')
    terms = []
    for i in range(0, len(parts)):
        if i % 2 == 0:
            # Unquoted text
            terms.extend([term for term in parts[i].split() if term])
        else:
            # The uneven parts are those inside quotes
            if parts[i]:
                terms.append('"%s"' % parts[i])
    return terms


def make_query(phrase):
    phrase = phrase.strip()
    if isinstance(phrase, str):
        phrase = phrase.decode('utf8')
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ISolrSettings)
    if is_simple_search(phrase):
        terms = split_simple_search(phrase)[:10]
        pattern = settings.simple_search_term_pattern
        term_queries = [pattern.format(term=escape(t)) for t in terms]
        if len(term_queries) > 1:
            term_queries = [u'(%s)' % q for q in term_queries]
        query = u' AND '.join(term_queries)
        if len(terms) > 1 or not phrase.isalnum():
            query = u'(%s) OR (%s)' % (
                settings.simple_search_phrase_pattern.format(
                    phrase=escape(phrase)),
                query,
            )
    else:
        pattern = settings.complex_search_pattern
        query = pattern.format(phrase=phrase)
    if settings.local_query_parameters:
        query = settings.local_query_parameters + query
    return query



# -*- coding: utf-8 -*-
from DateTime import DateTime
from ftw.solr.connection import SolrResponse
from ftw.solr.handlers import DefaultIndexHandler
from ftw.solr.schema import SolrSchema
from ftw.solr.testing import FTW_SOLR_INTEGRATION_TESTING
from ftw.solr.tests.utils import get_data
from mock import MagicMock
from mock import PropertyMock
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.uuid.interfaces import IMutableUUID
from Products.CMFPlone.utils import base_hasattr
import unittest


class TestDefaultIndexHandler(unittest.TestCase):

    layer = FTW_SOLR_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.doc = api.content.create(
            type='Document', title='My Document', id='doc',
            container=self.portal)
        if base_hasattr(self.doc, '_setUID'):
            self.doc._setUID('09baa75b67f44383880a6dab8b3200b6')
            self.doc.setModificationDate(DateTime('2017-01-21T17:18:19+00:00'))
        else:
            IMutableUUID(self.doc).set('09baa75b67f44383880a6dab8b3200b6')
            self.doc.modification_date = DateTime('2017-01-21T17:18:19+00:00')

        conn = MagicMock(name='SolrConnection')
        conn.get = MagicMock(name='get', return_value=SolrResponse(
            body=get_data('schema.json'), status=200))
        self.manager = MagicMock(name='SolrConnectionManager')
        type(self.manager).connection = PropertyMock(return_value=conn)
        type(self.manager).schema = PropertyMock(return_value=SolrSchema(
            self.manager))
        self.handler = DefaultIndexHandler(self.doc, self.manager)

    def test_get_data_without_attributes_gets_all_fields(self):
        self.assertEqual(
            {
                u'Title': u'My Document',
                u'UID': u'09baa75b67f44383880a6dab8b3200b6',
                u'allowedRolesAndUsers': [u'Anonymous'],
                u'path': u'/plone/doc',
                u'modified': u'2017-01-21T17:18:19.000Z',
                u'SearchableText': 'doc  My Document ',
            },
            self.handler.get_data(None)
        )

    def test_get_data_gets_only_specified_attributes(self):
        self.assertEqual(
            {
                u'UID': u'09baa75b67f44383880a6dab8b3200b6',
                u'path': u'/plone/doc',
            },
            self.handler.get_data(['UID', 'path'])
        )

    def test_get_data_always_includes_unique_key(self):
        self.assertEqual(
            {
                u'Title': u'My Document',
                u'UID': u'09baa75b67f44383880a6dab8b3200b6',
            },
            self.handler.get_data(['Title'])
        )

    def test_add_without_attributes_adds_full_documemt(self):
        self.manager.connection.add = MagicMock(name='add')
        self.handler.add(None)
        self.manager.connection.add.assert_called_once_with({
            u'UID': u'09baa75b67f44383880a6dab8b3200b6',
            u'Title': u'My Document',
            u'modified': u'2017-01-21T17:18:19.000Z',
            u'SearchableText': 'doc  My Document ',
            u'allowedRolesAndUsers': [u'Anonymous'],
            u'path': u'/plone/doc',
        })

    def test_add_with_attributes_does_atomic_update(self):
        self.manager.connection.add = MagicMock(name='add')
        self.handler.add(['Title', 'path'])
        self.manager.connection.add.assert_called_once_with({
            u'UID': u'09baa75b67f44383880a6dab8b3200b6',
            u'Title': {'set': u'My Document'},
            u'path': {'set': u'/plone/doc'},
        })

    def test_delete(self):
        self.manager.connection.delete = MagicMock(name='delete')
        self.handler.delete()
        self.manager.connection.delete.assert_called_once_with(
            u'09baa75b67f44383880a6dab8b3200b6')



from Products.CMFCore.utils import getToolByName
from ftw.solr.converters import CONVERTERS
from ftw.solr.interfaces import ISolrIndexHandler
from logging import getLogger
from plone.indexer.interfaces import IIndexableObject
from plone.namedfile.interfaces import INamedBlobFile
from plone.rfc822.interfaces import IPrimaryFieldInfo
from zope.component import queryMultiAdapter
from zope.interface import implementer


logger = getLogger('ftw.solr.handlers')


@implementer(ISolrIndexHandler)
class DefaultIndexHandler(object):

    def __init__(self, context, manager):
        self.context = context
        self.manager = manager

    def add(self, attributes):
        conn = self.manager.connection
        if conn is None:
            return

        schema = self.manager.schema
        if not schema:
            logger.warning(
                'Unable to fetch schema, skipping indexing of %r',
                self.context)
            return

        unique_key = schema.unique_key
        if unique_key is None:
            logger.warning(
                'Schema is missing unique key, skipping indexing of %r',
                self.context)
            return

        data = self.get_data(attributes)

        if unique_key not in data:
            logger.warning(
                'Object is missing unique key, skipping indexing of %r',
                self.context)
            return

        # Atomic update: add set modifier except for unique key
        if attributes:
            id_ = data[unique_key]
            data = {k: {'set': v} for k, v in data.items() if k != unique_key}
            data[unique_key] = id_

        conn.add(data)

    def delete(self):
        conn = self.manager.connection
        if conn is None:
            return

        schema = self.manager.schema
        if not schema:
            logger.warning(
                'Unable to fetch schema, skipping unindexing of %r',
                self.context)
            return

        unique_key = schema.unique_key
        if unique_key is None:
            logger.warning(
                'Schema is missing unique key, skipping unindexing of %r',
                self.context)
            return

        data = self.get_data([unique_key])
        if unique_key not in data:
            logger.warning(
                'Object is missing unique key, skipping unindexing of %r',
                self.context)

        conn.delete(data[unique_key])

    def get_data(self, attributes):
        if not IIndexableObject.providedBy(self.context):
            catalog = getToolByName(self.context, 'portal_catalog')
            wrapped = queryMultiAdapter(
                (self.context, catalog), IIndexableObject,
                default=self.context)
        else:
            wrapped = self.context

        schema = self.manager.schema
        if attributes is None:
            attributes = set(schema.fields.keys())
        else:
            attributes = set(schema.fields.keys()).intersection(attributes)
        if schema.unique_key not in attributes:
            attributes.add(schema.unique_key)

        data = {}
        for name in attributes:
            if name.startswith('_'):
                continue
            if name == 'path':
                value = '/'.join(self.context.getPhysicalPath())
            else:
                try:
                    value = getattr(wrapped, name)
                    if callable(value):
                        value = value()
                except (AttributeError, TypeError):
                    continue

            if value is None:
                continue
            field_class = schema.field_types[
                schema.fields[name][u'type']][u'class']
            multivalued = schema.fields[name].get(u'multiValued', False)
            converter = CONVERTERS.get(field_class)
            if converter is not None:
                value = converter(value, multivalued)
            data[name] = value

        return data


@implementer(ISolrIndexHandler)
class ATBlobFileIndexHandler(DefaultIndexHandler):

    def add(self, attributes):
        conn = self.manager.connection
        if conn is None:
            return

        if attributes and 'SearchableText' not in attributes:
            return super(ATBlobFileIndexHandler, self).add(attributes)

        schema = self.manager.schema
        if not schema:
            logger.warning(
                'Unable to fetch schema, skipping indexing of %r',
                self.context)
            return

        if attributes is None:
            attributes = schema.fields.keys()
        attributes.remove('SearchableText')

        data = self.get_data(attributes)
        field = self.context.getPrimaryField()
        blob = field.get(self.context).blob

        conn.extract(blob, data)


@implementer(ISolrIndexHandler)
class DexterityItemIndexHandler(DefaultIndexHandler):

    def add(self, attributes):
        conn = self.manager.connection
        if conn is None:
            return

        try:
            info = IPrimaryFieldInfo(self.context, None)
        except TypeError:
            info = None
        if info is not None and INamedBlobFile.providedBy(info.value):
            blob = info.value._blob
        else:
            return super(DexterityItemIndexHandler, self).add(attributes)

        if attributes and 'SearchableText' not in attributes:
            return super(DexterityItemIndexHandler, self).add(attributes)

        schema = self.manager.schema
        if not schema:
            logger.warning(
                'Unable to fetch schema, skipping indexing of %r',
                self.context)
            return
        if attributes is None:
            attributes = schema.fields.keys()
        attributes.remove('SearchableText')

        data = self.get_data(attributes)
        conn.extract(blob, data)



# -*- coding: utf-8 -*-
from DateTime import DateTime
from ftw.solr.connection import SolrResponse
from ftw.solr.contentlisting import SolrContentListing
from ftw.solr.contentlisting import SolrContentListingObject
from ftw.solr.contentlisting import SolrDocument
from ftw.solr.testing import FTW_SOLR_INTEGRATION_TESTING
from ftw.solr.tests.utils import get_data
from plone.app.contentlisting.interfaces import IContentListing
from plone.app.contentlisting.interfaces import IContentListingObject
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import json
import unittest


class TestContentListing(unittest.TestCase):

    layer = FTW_SOLR_INTEGRATION_TESTING

    def setUp(self):
        self.resp = SolrResponse(body=get_data('search.json'), status=200)
        self.contentlisting = SolrContentListing(self.resp)

    def test_contentlisting_creation(self):
        contentlisting = IContentListing(self.resp)
        self.assertTrue(isinstance(contentlisting, SolrContentListing))

    def test_contentlisting_actual_result_count(self):
        self.assertEqual(self.contentlisting.actual_result_count, 3)

    def test_contentlisting_getitem(self):
        self.assertTrue(
            IContentListingObject.providedBy(self.contentlisting[0]))

    def test_contentlisting_iteration(self):
        self.assertTrue(IContentListingObject.providedBy(
            [item for item in self.contentlisting][0]))


class TestContentListingObject(unittest.TestCase):

    layer = FTW_SOLR_INTEGRATION_TESTING

    def setUp(self):
        self.obj = SolrContentListingObject(SolrDocument(
            json.loads(get_data('doc.json'))))

    def test_idublincore_title(self):
        self.assertEqual(self.obj.Title(), 'My Document')

    def test_idublincore_description(self):
        self.assertEqual(self.obj.Description(), 'Foo bar baz.')

    def test_idublincore_type(self):
        self.assertEqual(self.obj.Type(), 'Document')

    def test_idublincore_created(self):
        self.assertEqual(
            self.obj.created(), DateTime('2017/08/01 13:17:0.009000 GMT+2'))

    def test_idublincore_modified(self):
        self.assertEqual(
            self.obj.modified(), DateTime('2017/12/31 12:17:0.137000 GMT+1'))

    def test_idublincore_effective(self):
        self.assertEqual(
            self.obj.effective(), None)

    def test_idublincore_expires(self):
        self.assertEqual(
            self.obj.expires(), None)

    def test_idublincore_creationdate(self):
        self.assertEqual(
            self.obj.CreationDate(), '2017-08-01T13:17:00+02:00')

    def test_idublincore_modificationdate(self):
        self.assertEqual(
            self.obj.ModificationDate(), '2017-12-31T12:17:00+01:00')

    def test_idublincore_effectivedate(self):
        self.assertEqual(
            self.obj.EffectiveDate(), None)

    def test_idublincore_expirationdate(self):
        self.assertEqual(
            self.obj.ExpirationDate(), None)

    def test_icontentlistingobject_getid(self):
        self.assertEqual(self.obj.getId(), 'my-document.docx')

    def test_icontentlistingobject_getobject(self):
        portal = self.layer['portal']
        setRoles(portal, TEST_USER_ID, ['Manager'])
        portal.invokeFactory('Document', 'my-document')
        obj = portal['my-document']
        self.obj.doc.data[u'path'] = '/'.join(obj.getPhysicalPath())
        self.assertEqual(self.obj.getObject(), obj)

    def test_icontentlistingobject_getobject_for_not_existing_object(self):
        self.assertEqual(self.obj.getObject(), None)

    def test_icontentlistingobject_getdataorigin(self):
        self.assertTrue(isinstance(self.obj.getDataOrigin(), SolrDocument))

    def test_icontentlistingobject_getpath(self):
        self.assertEqual(
            self.obj.getPath(), '/plone/my-folder-1/my-document.docx')

    def test_icontentlistingobject_geturl(self):
        self.assertEqual(
            self.obj.getURL(),
            'http://nohost/plone/my-folder-1/my-document.docx')

    def test_icontentlistingobject_uuid(self):
        self.assertEqual(
            self.obj.uuid(), '9398dad21bcd49f8a197cd50d10ea778')

    def test_icontentlistingobject_geticon(self):
        self.assertEqual(
            self.obj.getIcon(),
            '<img width="16" height="16" '
            'src="http://nohost/plone/icon_dokument_word.gif" '
            'alt="Page Word 2007 document" />')

    def test_icontentlistingobject_getsize(self):
        self.assertEqual(self.obj.getSize(), '374 KB')

    def test_icontentlistingobject_review_state(self):
        self.assertEqual(self.obj.review_state(), 'private')

    def test_icontentlistingobject_portaltype(self):
        self.assertEqual(self.obj.PortalType(), 'Document')

    def test_icontentlistingobject_croppeddescription(self):
        self.assertEqual(self.obj.CroppedDescription(), 'Foo bar baz.')

    def test_icontentlistingobject_contenttypeclass(self):
        self.assertEqual(self.obj.ContentTypeClass(), 'contenttype-document')



# -*- coding: utf-8 -*-
from ftw.solr.interfaces import ISolrSettings
from ftw.solr.query import escape
from ftw.solr.query import is_simple_search
from ftw.solr.query import make_query
from ftw.solr.query import split_simple_search
from plone.registry import Registry
from plone.registry.fieldfactory import persistentFieldAdapter
from plone.registry.interfaces import IRegistry
from plone.testing import zca
from zope.component import provideAdapter
from zope.component import provideUtility
import unittest


class TestQueryHelpers(unittest.TestCase):

    def test_escape(self):
        self.assertEqual(escape('foo!'), 'foo\\!')
        self.assertEqual(escape('1:1'), '1\\:1')
        self.assertEqual(escape('/path/to/a/file'), '\\/path\\/to\\/a\\/file')

    def test_is_simple_search(self):
        self.assertTrue(is_simple_search('foo'))
        self.assertTrue(is_simple_search('foo bar'))
        self.assertTrue(is_simple_search('"foo bar"'))
        self.assertTrue(is_simple_search('foo "bar foobar" baz'))
        self.assertTrue(is_simple_search('"foo AND bar"'))
        self.assertTrue(is_simple_search('foo "AND" bar'))
        self.assertTrue(is_simple_search('foo"'))

        self.assertFalse(is_simple_search('foo AND bar'))
        self.assertFalse(is_simple_search('foo OR bar'))
        self.assertFalse(is_simple_search('"foo bar" OR baz'))

    def test_split_simple_search(self):
        self.assertEqual(split_simple_search('foo'), ['foo'])
        self.assertEqual(split_simple_search('foo bar'), ['foo', 'bar'])
        self.assertEqual(split_simple_search('"foo bar"'), ['"foo bar"'])
        self.assertEqual(
            split_simple_search('foo "bar foobar" baz'),
            ['foo', '"bar foobar"', 'baz'])


class TestMakeQuery(unittest.TestCase):

    layer = zca.UNIT_TESTING

    def setUp(self):
        provideAdapter(persistentFieldAdapter)
        registry = Registry()
        registry.registerInterface(ISolrSettings)
        provideUtility(registry, IRegistry)
        self.settings = registry.forInterface(ISolrSettings)
        self.settings.simple_search_term_pattern = (
            u'Title:{term}^10 OR SearchableText:{term} OR '
            u'SearchableText:{term}*')
        self.settings.simple_search_phrase_pattern = (
            u'Title:"{phrase}"^20 OR SearchableText:"{phrase}"^5 OR '
            u'SearchableText:"{phrase}*"^2')
        self.settings.complex_search_pattern = (
            u'Title:({phrase})^10 OR SearchableText:({phrase})')
        self.settings.local_query_parameters = u''

    def test_single_search_word(self):
        self.assertEqual(
            make_query('foo'),
            u'Title:foo^10 OR SearchableText:foo OR SearchableText:foo*')

    def test_multiple_search_words(self):
        self.assertEqual(
            make_query('foo bar baz'),
            u'(Title:"foo bar baz"^20 OR SearchableText:"foo bar baz"^5 OR '
            u'SearchableText:"foo bar baz*"^2) OR ('
            u'(Title:foo^10 OR SearchableText:foo OR SearchableText:foo*) AND '
            u'(Title:bar^10 OR SearchableText:bar OR SearchableText:bar*) AND '
            u'(Title:baz^10 OR SearchableText:baz OR SearchableText:baz*))')

    def test_search_words_with_binary_operator(self):
        self.assertEqual(
            make_query('foo AND bar'),
            u'Title:(foo AND bar)^10 OR SearchableText:(foo AND bar)')

    def test_search_word_with_special_chars(self):
        self.assertEqual(
            make_query('C++'),
            u'(Title:"C\\+\\+"^20 OR SearchableText:"C\\+\\+"^5 OR '
            u'SearchableText:"C\\+\\+*"^2) OR '
            u'(Title:C\\+\\+^10 OR SearchableText:C\\+\\+ OR '
            u'SearchableText:C\\+\\+*)')

    def test_query_with_local_parameters(self):
        self.settings.local_query_parameters = (
            u'{!boost b=recip(ms(NOW,modified),3.858e-10,10,1)}')
        self.assertEqual(
            make_query('foo'),
            u'{!boost b=recip(ms(NOW,modified),3.858e-10,10,1)}'
            u'Title:foo^10 OR SearchableText:foo OR SearchableText:foo*')

    def test_long_phrase_gets_truncated_to_10_terms(self):
        self.settings.simple_search_term_pattern = u'Title:{term}'
        self.settings.simple_search_phrase_pattern = u'Title:"{phrase}"'
        self.assertEqual(
            make_query(
                'one two three four five six seven eight nine ten eleven'),
            u'(Title:"one two three four five six seven eight nine ten '
            u'eleven") OR ((Title:one) AND (Title:two) AND (Title:three) AND '
            u'(Title:four) AND (Title:five) AND (Title:six) AND (Title:seven) '
            u'AND (Title:eight) AND (Title:nine) AND (Title:ten))')

    def test_query_string_is_unicode(self):
        self.assertTrue(isinstance(make_query('über'), unicode))
        self.assertTrue(isinstance(make_query(u'über'), unicode))
